require File.expand_path('../test_helper', __FILE__)

describe Elastomer::CLI::Cluster do
  describe 'health' do
    it 'should get cluster health' do
      VCR.use_cassette('default') do
        elastomer 'cluster health'
        @out.must_match /CLUSTER HEALTH.*Bug/
        @out.must_match /Name.*elasticsearch/
        @out.must_match /Status.*green/
        @out.must_match /Timed Out.*false/
        @out.must_match /Number of Nodes.*1/
        @out.must_match /Number of Data Nodes.*1/
        @out.must_match /Active Primary Shards.*0/
        @out.must_match /Active Shards.*0/
        @out.must_match /Relocating Shards.*0/
        @out.must_match /Initializing Shards.*0/
        @out.must_match /Unassigned Shards.*0/
      end
    end
  end

  describe 'settings' do
    it 'should get persistent and transient cluster settings' do
      VCR.use_cassette('default') do
        elastomer 'cluster settings'
        @out.must_match /PERSISTENT/
        @out.must_match /indices\.ttl\.interval\s+\|\s+50/
        @out.must_match /TRANSIENT/
        @out.must_match /indices\.ttl\.interval\s+\|\s+60/
      end
    end
  end

  describe 'update' do
    it 'should allow updating transient settings' do
      VCR.use_cassette('default') do
        elastomer 'cluster update indices.ttl.interval=70'
        @out.wont_match /PERSISTENT/
        @out.must_match /TRANSIENT/
        @out.must_match /indices\.ttl\.interval=70/
        @out.must_match /indices\.ttl\.interval\s+\|\s+70/
      end
    end

    it 'should allow updating multiple transient settings' do
      VCR.use_cassette('default') do
        elastomer 'cluster update indices.ttl.interval=70 cluster.routing.allocation.enable=all'
        @out.wont_match /PERSISTENT/
        @out.must_match /TRANSIENT/
        @out.must_match /indices\.ttl\.interval=70/
        @out.must_match /cluster\.routing\.allocation\.enable=all/
        @out.must_match /indices\.ttl\.interval\s+\|\s+70/
        @out.must_match /cluster\.routing\.allocation\.enable\s+\|\s+all/
      end
    end

    it 'should allow updating persistent settings' do
      VCR.use_cassette('default') do
        elastomer 'cluster update indices.ttl.interval=70 --persistent'
        @out.must_match /PERSISTENT/
        @out.wont_match /TRANSIENT/
        @out.must_match /indices\.ttl\.interval=70/
        @out.must_match /indices\.ttl\.interval\s+\|\s+70/
      end
    end

    it 'should allow updating multiple persistent settings' do
      VCR.use_cassette('default') do
        elastomer 'cluster update indices.ttl.interval=70 cluster.routing.allocation.enable=all --persistent'
        @out.must_match /PERSISTENT/
        @out.wont_match /TRANSIENT/
        @out.must_match /indices\.ttl\.interval=70/
        @out.must_match /cluster\.routing\.allocation\.enable=all/
        @out.must_match /indices\.ttl\.interval\s+\|\s+70/
        @out.must_match /cluster\.routing\.allocation\.enable\s+\|\s+all/
      end
    end
  end

  describe 'allocation' do

    it 'should deny unknown commands' do
      VCR.use_cassette('default') do
        elastomer 'cluster allocation foobar'
        @err.must_match "ERROR: Unknown allocation command: foobar"
      end
    end

    it 'should deny unknown versions' do
      VCR.use_cassette('0.20') do
        elastomer 'cluster allocation disable'
        @err.must_match "ERROR: Unknown Elasticsearch version: 0.20.6"
      end
    end

    describe '0.90' do
      it 'should disable allocation' do
        VCR.use_cassette('0.90') do
          elastomer 'cluster allocation disable'
          @out.wont_match /PERSISTENT/
          @out.must_match /TRANSIENT/
          @out.must_match /cluster\.routing\.allocation\.disable_allocation=true/
          @out.must_match /cluster\.routing\.allocation.disable_allocation\s+\|\s+true/
        end
      end

      it 'should enable allocation' do
        VCR.use_cassette('0.90') do
          elastomer 'cluster allocation enable'
          @out.wont_match /PERSISTENT/
          @out.must_match /TRANSIENT/
          @out.must_match /cluster\.routing\.allocation\.disable_allocation=false/
          @out.must_match /cluster\.routing\.allocation.disable_allocation\s+\|\s+false/
        end
      end
    end

    describe '1.x' do
      it 'should disable allocation' do
        VCR.use_cassette('1.x') do
          elastomer 'cluster allocation disable'
          @out.wont_match /PERSISTENT/
          @out.must_match /TRANSIENT/
          @out.must_match /cluster\.routing\.allocation\.enable=none/
          @out.must_match /cluster\.routing\.allocation\.enable\s+\|\s+none/
        end
      end

      it 'should enable allocation' do
        VCR.use_cassette('1.x') do
          elastomer 'cluster allocation enable'
          @out.wont_match /PERSISTENT/
          @out.must_match /TRANSIENT/
          @out.must_match /cluster\.routing\.allocation\.enable=all/
          @out.must_match /cluster\.routing\.allocation\.enable\s+\|\s+all/
        end
      end

      it 'should allow all allocation states' do
        VCR.use_cassette('1.x') do
          %w{ all primaries new_primaries none }.each do |state|
            elastomer "cluster allocation #{state}"
            @out.wont_match /PERSISTENT/
            @out.must_match /TRANSIENT/
            @out.must_match /cluster\.routing\.allocation\.enable=#{state}/
            @out.must_match /cluster\.routing\.allocation\.enable\s+\|\s+#{state}/
          end
        end
      end
    end
  end
end
